@@ -54,6 +54,8 @@
 // Extra padding at end of buffer.
 #define BUFFER_EXTRA 10
 
+#define MAX_DEPTH 1000
+
 typedef unsigned long	ulong;
 
 static void	raise_strict(VALUE obj);
@@ -68,16 +70,16 @@ static void	dump_raw(const char *str, size_t cnt, Out out);
 static void	dump_cstr(const char *str, size_t cnt, int is_sym, int escape1, Out out);
 static void	dump_hex(uint8_t c, Out out);
 static void	dump_str_comp(VALUE obj, Out out);
-static void	dump_str_obj(VALUE obj, Out out);
+static void	dump_str_obj(VALUE obj, VALUE clas, int depth, Out out);
 static void	dump_sym_comp(VALUE obj, Out out);
 static void	dump_sym_obj(VALUE obj, Out out);
 static void	dump_class_comp(VALUE obj, Out out);
 static void	dump_class_obj(VALUE obj, Out out);
-static void	dump_array(VALUE obj, int depth, Out out);
+static void	dump_array(VALUE obj, VALUE clas, int depth, Out out);
 static int	hash_cb_strict(VALUE key, VALUE value, Out out);
 static int	hash_cb_compat(VALUE key, VALUE value, Out out);
 static int	hash_cb_object(VALUE key, VALUE value, Out out);
-static void	dump_hash(VALUE obj, int depth, int mode, Out out);
+static void	dump_hash(VALUE obj, VALUE clas, int depth, int mode, Out out);
 static void	dump_time(VALUE obj, Out out);
 static void	dump_ruby_time(VALUE obj, Out out);
 static void	dump_xml_time(VALUE obj, Out out);
@@ -107,11 +109,6 @@ static void	dump_leaf_float(Leaf leaf, Out out);
 static void	dump_leaf_array(Leaf leaf, int depth, Out out);
 static void	dump_leaf_hash(Leaf leaf, int depth, Out out);
 
-// These are used to detect rails re-call of Oj.dump() inside the to_json()
-// method. It is not thread safe.
-static VALUE	last_obj = Qundef;
-static int	oj_rails_hack = -1;
-
 static const char	hex_chars[17] = "0123456789abcdef";
 
 static char	hibit_friendly_chars[256] = "\
@@ -187,19 +184,6 @@ fill_indent(Out out, int cnt) {
     }
 }
 
-inline static const char*
-ulong2str(uint32_t num, char *end) {
-    char	*b;
-
-    *end-- = '\0';
-    for (b = end; 0 < num || b == end; num /= 10, b--) {
-	*b = (num % 10) + '0';
-    }
-    b++;
-
-    return b;
-}
-
 inline static void
 dump_ulong(unsigned long num, Out out) {
     char	buf[32];
@@ -561,12 +545,16 @@ dump_str_comp(VALUE obj, Out out) {
 }
 
 static void
-dump_str_obj(VALUE obj, Out out) {
-    const char	*s = rb_string_value_ptr((VALUE*)&obj);
-    size_t	len = RSTRING_LEN(obj);
-    char	s1 = s[1];
+dump_str_obj(VALUE obj, VALUE clas, int depth, Out out) {
+    if (Qundef != clas && rb_cString != clas) {
+	dump_obj_attrs(obj, clas, 0, depth, out);
+    } else {
+	const char	*s = rb_string_value_ptr((VALUE*)&obj);
+	size_t		len = RSTRING_LEN(obj);
+	char		s1 = s[1];
 
-    dump_cstr(s, len, 0, (':' == *s || ('^' == *s && ('r' == s1 || 'i' == s1))), out);
+	dump_cstr(s, len, 0, (':' == *s || ('^' == *s && ('r' == s1 || 'i' == s1))), out);
+    }
 }
 
 static void
@@ -611,7 +599,7 @@ dump_class_obj(VALUE obj, Out out) {
 }
 
 static void
-dump_array(VALUE a, int depth, Out out) {
+dump_array(VALUE a, VALUE clas, int depth, Out out) {
     size_t	size;
     int		i, cnt;
     int		d2 = depth + 1;
@@ -620,6 +608,10 @@ dump_array(VALUE a, int depth, Out out) {
     if (id < 0) {
 	return;
     }
+    if (Qundef != clas && rb_cArray != clas) {
+	dump_obj_attrs(a, clas, 0, depth, out);
+	return;
+    }
     cnt = (int)RARRAY_LEN(a);
     *out->cur++ = '[';
     if (0 < id) {
@@ -828,7 +820,7 @@ hash_cb_object(VALUE key, VALUE value, Out out) {
     }
     fill_indent(out, depth);
     if (rb_type(key) == T_STRING) {
-	dump_str_obj(key, out);
+	dump_str_obj(key, Qundef, depth, out);
 	*out->cur++ = ':';
 	dump_val(value, depth, out);
     } else if (rb_type(key) == T_SYMBOL) {
@@ -882,10 +874,16 @@ hash_cb_object(VALUE key, VALUE value, Out out) {
 }
 
 static void
-dump_hash(VALUE obj, int depth, int mode, Out out) {
-    int		cnt = (int)RHASH_SIZE(obj);
-    size_t	size = depth * out->indent + 2;
+dump_hash(VALUE obj, VALUE clas, int depth, int mode, Out out) {
+    int		cnt;
+    size_t	size;
 
+    if (Qundef != clas && rb_cHash != clas && ObjectMode == mode) {
+	dump_obj_attrs(obj, clas, 0, depth, out);
+	return;
+    }
+    cnt = (int)RHASH_SIZE(obj);
+    size = depth * out->indent + 2;
     if (out->end - out->cur <= 2) {
 	grow(out, 2);
     }
@@ -1153,17 +1151,15 @@ dump_data_comp(VALUE obj, int depth, Out out) {
 	if (T_HASH != rb_type(h)) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
-	dump_hash(h, depth, out->opts->mode, out);
+	dump_hash(h, Qundef, depth, out->opts->mode, out);
     } else if (rb_respond_to(obj, oj_as_json_id) && obj != (o2 = rb_funcall(obj, oj_as_json_id, 0))) {
 	dump_val(o2, depth, out);
-    } else if (rb_respond_to(obj, oj_to_json_id) && (!oj_rails_hack || last_obj != obj)) {
+    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
 	const char	*s;
 	int		len;
 
-	last_obj = obj;
 	rs = rb_funcall(obj, oj_to_json_id, 0);
-	last_obj = Qundef;
 	s = rb_string_value_ptr((VALUE*)&rs);
 	len = (int)RSTRING_LEN(rs);
 
@@ -1217,22 +1213,16 @@ dump_data_obj(VALUE obj, int depth, Out out) {
 	*out->cur++ = '}';
 	*out->cur = '\0';
     } else {
-	Odd	odd = oj_get_odd(clas);
-
-	if (0 == odd) {
-	    if (oj_bigdecimal_class == clas) {
-		volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+	if (oj_bigdecimal_class == clas) {
+	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 		
-		if (Yes == out->opts->bigdec_as_num) {
-		    dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
-		} else {
-		    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
-		}
+	    if (Yes == out->opts->bigdec_as_num) {
+		dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
 	    } else {
-		dump_nil(out);
+		dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	    }
 	} else {
-	    dump_odd(obj, odd, clas, depth + 1, out);
+	    dump_nil(out);
 	}
     }
 }
@@ -1245,17 +1235,15 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 	if (T_HASH != rb_type(h)) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
-	dump_hash(h, depth, out->opts->mode, out);
+	dump_hash(h, Qundef, depth, out->opts->mode, out);
     } else if (rb_respond_to(obj, oj_as_json_id)) {
 	dump_val(rb_funcall(obj, oj_as_json_id, 0), depth, out);
-    } else if (rb_respond_to(obj, oj_to_json_id) && (!oj_rails_hack || last_obj != obj)) {
+    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
 	const char	*s;
 	int		len;
 
-	last_obj = obj;
 	rs = rb_funcall(obj, oj_to_json_id, 0);
-	last_obj = Qundef;
 	s = rb_string_value_ptr((VALUE*)&rs);
 	len = (int)RSTRING_LEN(rs);
 
@@ -1281,13 +1269,7 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 
 	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	} else {
-	    Odd	odd = oj_get_odd(clas);
-
-	    if (0 == odd) {
-		dump_obj_attrs(obj, 0, 0, depth, out);
-	    } else {
-		dump_odd(obj, odd, 0, depth + 1, out);
-	    }
+	    dump_obj_attrs(obj, Qundef, 0, depth, out);
 	}
     }
     *out->cur = '\0';
@@ -1299,22 +1281,40 @@ dump_obj_obj(VALUE obj, int depth, Out out) {
 
     if (0 <= id) {
 	VALUE	clas = rb_obj_class(obj);
-	Odd	odd = oj_get_odd(clas);
 
-	if (0 == odd) {
-	    if (oj_bigdecimal_class == clas) {
-		volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+	if (oj_bigdecimal_class == clas) {
+	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 
-		dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
-	    } else {
-		dump_obj_attrs(obj, clas, id, depth, out);
-	    }
+	    dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
 	} else {
-	    dump_odd(obj, odd, clas, depth + 1, out);
+	    dump_obj_attrs(obj, clas, id, depth, out);
 	}
     }
 }
 
+#ifdef RUBINIUS_RUBY
+static int
+isRbxHashAttr(const char *attr) {
+    const char	*hashAttrs[] = {
+	"@capacity",
+	"@max_entries",
+	"@state",
+	"@mask",
+	"@size",
+	"@entries",
+	"@default_proc",
+	0 };
+    const char	**ap;
+
+    for (ap = hashAttrs; 0 != *ap; ap++) {
+	if (0 == strcmp(attr, *ap)) {
+	    return 1;
+	}
+    }
+    return 0;
+}
+#endif
+
 #if HAS_IVAR_HELPERS
 static int
 dump_attr_cb(ID key, VALUE value, Out out) {
@@ -1355,12 +1355,13 @@ static void
 dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
     size_t	size = 0;
     int		d2 = depth + 1;
+    int		type = rb_type(obj);
 
     if (out->end - out->cur <= 2) {
 	grow(out, 2);
     }
     *out->cur++ = '{';
-    if (0 != clas) {
+    if (Qundef != clas) {
 	const char	*class_name = rb_class2name(clas);
 	int		clen = (int)strlen(class_name);
 
@@ -1390,6 +1391,58 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 	*out->cur++ = ':';
 	dump_ulong(id, out);
     }
+    switch (type) {
+    case T_STRING:
+	size = d2 * out->indent + 14;
+	if (out->end - out->cur <= (long)size) {
+	    grow(out, size);
+	}
+	*out->cur++ = ',';
+	fill_indent(out, d2);
+	*out->cur++ = '"';
+	*out->cur++ = 's';
+	*out->cur++ = 'e';
+	*out->cur++ = 'l';
+	*out->cur++ = 'f';
+	*out->cur++ = '"';
+	*out->cur++ = ':';
+	dump_cstr(rb_string_value_ptr((VALUE*)&obj), RSTRING_LEN(obj), 0, 0, out);
+	break;
+    case T_ARRAY:
+	size = d2 * out->indent + 14;
+	if (out->end - out->cur <= (long)size) {
+	    grow(out, size);
+	}
+	*out->cur++ = ',';
+	fill_indent(out, d2);
+	*out->cur++ = '"';
+	*out->cur++ = 's';
+	*out->cur++ = 'e';
+	*out->cur++ = 'l';
+	*out->cur++ = 'f';
+	*out->cur++ = '"';
+	*out->cur++ = ':';
+	dump_array(obj, Qundef, depth + 1, out);
+	break;
+    case T_HASH:
+	size = d2 * out->indent + 14;
+	if (out->end - out->cur <= (long)size) {
+	    grow(out, size);
+	}
+	*out->cur++ = ',';
+	fill_indent(out, d2);
+	*out->cur++ = '"';
+	*out->cur++ = 's';
+	*out->cur++ = 'e';
+	*out->cur++ = 'l';
+	*out->cur++ = 'f';
+	*out->cur++ = '"';
+	*out->cur++ = ':';
+	dump_hash(obj, Qundef, depth + 1, out->opts->mode, out);
+	break;
+    default:
+	break;
+    }
     {
 	int	cnt;
 #if HAS_IVAR_HELPERS
@@ -1400,10 +1453,11 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 	ID		vid;
 	const char	*attr;
 	int		i;
+	int		first = 1;
 
 	cnt = (int)RARRAY_LEN(vars);
 #endif
-	if (0 != clas && 0 < cnt) {
+	if (Qundef != clas && 0 < cnt) {
 	    *out->cur++ = ',';
 	}
 	out->depth = depth + 1;
@@ -1415,12 +1469,22 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 #else
 	size = d2 * out->indent + 1;
 	for (i = cnt; 0 < i; i--, np++) {
+	    vid = rb_to_id(*np);
+	    attr = rb_id2name(vid);
+#ifdef RUBINIUS_RUBY
+	    if (T_HASH == type && isRbxHashAttr(attr)) {
+		continue;
+	    }
+#endif
+	    if (first) {
+		first = 0;
+	    } else {
+		*out->cur++ = ',';
+	    }
 	    if (out->end - out->cur <= (long)size) {
 		grow(out, size);
 	    }
-	    vid = rb_to_id(*np);
 	    fill_indent(out, d2);
-	    attr = rb_id2name(vid);
 	    if ('@' == *attr) {
 		attr++;
 		dump_cstr(attr, strlen(attr), 0, 0, out);
@@ -1437,9 +1501,6 @@ dump_obj_attrs(VALUE obj, VALUE clas, slot_t id, int depth, Out out) {
 	    if (out->end - out->cur <= 2) {
 		grow(out, 2);
 	    }
-	    if (1 < i) {
-	      *out->cur++ = ',';
-	    }
 	}
 #endif
 #if HAS_EXCEPTION_MAGIC
@@ -1491,7 +1552,7 @@ dump_struct_comp(VALUE obj, int depth, Out out) {
 	if (T_HASH != rb_type(h)) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
-	dump_hash(h, depth, out->opts->mode, out);
+	dump_hash(h, Qundef, depth, out->opts->mode, out);
     } else if (rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs = rb_funcall(obj, oj_to_json_id, 0);
 	const char	*s;
@@ -1584,7 +1645,7 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 	grow(out, 2);
     }
     *out->cur++ = '{';
-    if (0 != clas) {
+    if (Qundef != clas) {
 	const char	*class_name = rb_class2name(clas);
 	int		clen = (int)strlen(class_name);
 
@@ -1629,20 +1690,24 @@ raise_strict(VALUE obj) {
 
 static void
 dump_val(VALUE obj, int depth, Out out) {
+    int	type = rb_type(obj);
+
+    if (MAX_DEPTH < depth) {
+	rb_raise(rb_eNoMemError, "Too deeply nested.\n");
+    }
     switch (rb_type(obj)) {
-    case T_NIL:		dump_nil(out);			break;
-    case T_TRUE:	dump_true(out);			break;
-    case T_FALSE:	dump_false(out);		break;
-    case T_FIXNUM:	dump_fixnum(obj, out);		break;
-    case T_FLOAT:	dump_float(obj, out);		break;
-    case T_BIGNUM:	dump_bignum(obj, out);		break;
-    case T_STRING:
+    case T_NIL:		dump_nil(out);				break;
+    case T_TRUE:	dump_true(out);				break;
+    case T_FALSE:	dump_false(out);			break;
+    case T_FIXNUM:	dump_fixnum(obj, out);			break;
+    case T_FLOAT:	dump_float(obj, out);			break;
+    case T_CLASS:
 	switch (out->opts->mode) {
-	case StrictMode:
-	case NullMode:
-	case CompatMode:	dump_str_comp(obj, out);	break;
+	case StrictMode:	raise_strict(obj);		break;
+	case NullMode:		dump_nil(out);			break;
+	case CompatMode:	dump_class_comp(obj, out);	break;
 	case ObjectMode:
-	default:		dump_str_obj(obj, out);		break;
+	default:		dump_class_obj(obj, out);	break;
 	}
 	break;
     case T_SYMBOL:
@@ -1654,39 +1719,6 @@ dump_val(VALUE obj, int depth, Out out) {
 	default:		dump_sym_obj(obj, out);		break;
 	}
 	break;
-    case T_ARRAY:	dump_array(obj, depth, out);		break;
-    case T_HASH:	dump_hash(obj, depth, out->opts->mode, out);	break;
-    case T_CLASS:
-	switch (out->opts->mode) {
-	case StrictMode:	raise_strict(obj);		break;
-	case NullMode:		dump_nil(out);			break;
-	case CompatMode:	dump_class_comp(obj, out);	break;
-	case ObjectMode:
-	default:		dump_class_obj(obj, out);	break;
-	}
-	break;
-#if (defined T_RATIONAL && defined RRATIONAL)
-    case T_RATIONAL:
-#endif
-    case T_OBJECT:
-	switch (out->opts->mode) {
-	case StrictMode:	dump_data_strict(obj, out);	break;
-	case NullMode:		dump_data_null(obj, out);	break;
-	case CompatMode:	dump_obj_comp(obj, depth, out);	break;
-	case ObjectMode:
-	default:		dump_obj_obj(obj, depth, out);	break;
-	}
-	break;
-    case T_DATA:
-	switch (out->opts->mode) {
-	case StrictMode:	dump_data_strict(obj, out);	break;
-	case NullMode:		dump_data_null(obj, out);	break;
-	case CompatMode:	dump_data_comp(obj, depth, out);break;
-	case ObjectMode:
-	default:		dump_data_obj(obj, depth, out);	break;
-	}
-	break;
-
     case T_STRUCT: // for Range
 	switch (out->opts->mode) {
 	case StrictMode:	raise_strict(obj);		break;
@@ -1696,23 +1728,70 @@ dump_val(VALUE obj, int depth, Out out) {
 	default:		dump_struct_obj(obj, depth, out);	break;
 	}
 	break;
-
-#if (defined T_COMPLEX && defined RCOMPLEX)
-    case T_COMPLEX:
-#endif
-    case T_REGEXP:
-	switch (out->opts->mode) {
-	case StrictMode:	raise_strict(obj);		break;
-	case NullMode:		dump_nil(out);			break;
-	case CompatMode:
-	case ObjectMode:
-	default:		dump_obj_comp(obj, depth, out);	break;
-	}
-	break;
     default:
-	rb_raise(rb_eNotImpError, "Failed to dump '%s' Object (%02x)\n",
-		 rb_class2name(rb_obj_class(obj)), rb_type(obj));
-	break;
+	// Most developers have enough sense not to subclass primitive types but
+	// since these classes could potentially be subclassed a check for odd
+	// classes is performed.
+	{
+	    VALUE	clas = rb_obj_class(obj);
+	    Odd		odd;
+
+	    if (ObjectMode == out->opts->mode && 0 != (odd = oj_get_odd(clas))) {
+		dump_odd(obj, odd, clas, depth + 1, out);
+		return;
+	    }
+	    switch (type) {
+	    case T_BIGNUM:		dump_bignum(obj, out);		break;
+	    case T_STRING:
+		switch (out->opts->mode) {
+		case StrictMode:
+		case NullMode:
+		case CompatMode:	dump_str_comp(obj, out);	break;
+		case ObjectMode:
+		default:		dump_str_obj(obj, clas, depth, out);	break;
+		}
+		break;
+	    case T_ARRAY:		dump_array(obj, clas, depth, out);	break;
+	    case T_HASH:		dump_hash(obj, clas, depth, out->opts->mode, out);	break;
+#if (defined T_RATIONAL && defined RRATIONAL)
+	    case T_RATIONAL:
+#endif
+	    case T_OBJECT:
+		switch (out->opts->mode) {
+		case StrictMode:	dump_data_strict(obj, out);	break;
+		case NullMode:		dump_data_null(obj, out);	break;
+		case CompatMode:	dump_obj_comp(obj, depth, out);	break;
+		case ObjectMode:
+		default:		dump_obj_obj(obj, depth, out);	break;
+		}
+		break;
+	    case T_DATA:
+		switch (out->opts->mode) {
+		case StrictMode:	dump_data_strict(obj, out);	break;
+		case NullMode:		dump_data_null(obj, out);	break;
+		case CompatMode:	dump_data_comp(obj, depth, out);break;
+		case ObjectMode:
+		default:		dump_data_obj(obj, depth, out);	break;
+		}
+		break;
+#if (defined T_COMPLEX && defined RCOMPLEX)
+	    case T_COMPLEX:
+#endif
+	    case T_REGEXP:
+		switch (out->opts->mode) {
+		case StrictMode:	raise_strict(obj);		break;
+		case NullMode:		dump_nil(out);			break;
+		case CompatMode:
+		case ObjectMode:
+		default:		dump_obj_comp(obj, depth, out);	break;
+		}
+		break;
+	    default:
+		rb_raise(rb_eNotImpError, "Failed to dump '%s' Object (%02x)\n",
+			 rb_class2name(rb_obj_class(obj)), rb_type(obj));
+		break;
+	    }
+	}
     }
 }
 
@@ -1731,10 +1810,18 @@ oj_dump_obj_to_json(VALUE obj, Options copts, Out out) {
 	oj_cache8_new(&out->circ_cache);
     }
     out->indent = copts->indent;
-    if (0 > oj_rails_hack) {
-	oj_rails_hack = (rb_const_defined_at(rb_cObject, rb_intern("ActiveSupport")));
-    }
     dump_val(obj, 0, out);
+    if (0 < out->indent) {
+	switch (*(out->cur - 1)) {
+	case ']':
+	case '}':
+	    grow(out, 1);
+	    *out->cur++ = '\n';
+	    *out->cur = '\0';
+	default:
+	    break;
+	}
+    }
     if (Yes == copts->circular) {
 	oj_cache8_delete(out->circ_cache);
     }
@@ -1778,6 +1865,7 @@ oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts) {
     ssize_t	size;
     VALUE	clas = rb_obj_class(stream);
 #if !IS_WINDOWS
+    int		fd;
     VALUE	s;
 #endif
 
@@ -1789,9 +1877,9 @@ oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts) {
     if (oj_stringio_class == clas) {
 	rb_funcall(stream, oj_write_id, 1, rb_str_new(out.buf, size));
 #if !IS_WINDOWS
-    } else if (rb_respond_to(stream, oj_fileno_id) && Qnil != (s = rb_funcall(stream, oj_fileno_id, 0))) {
-	int	fd = FIX2INT(s);
-
+    } else if (rb_respond_to(stream, oj_fileno_id) &&
+	       Qnil != (s = rb_funcall(stream, oj_fileno_id, 0)) &&
+	       0 != (fd = FIX2INT(s))) {
 	if (size != write(fd, out.buf, size)) {
 	    if (out.allocated) {
 		xfree(out.buf);
@@ -2069,10 +2157,16 @@ maybe_comma(StrWriter sw) {
 }
 
 void
-oj_str_writer_push_object(StrWriter sw, const char *key) {
+oj_str_writer_push_key(StrWriter sw, const char *key) {
+    DumpType	type = sw->types[sw->depth];
     long	size;
 
-    key_check(sw, key);
+    if (sw->keyWritten) {
+	rb_raise(rb_eStandardError, "Can not push more than one key before pushing a non-key.");
+    }
+    if (ObjectNew != type && ObjectType != type) {
+	rb_raise(rb_eStandardError, "Can only push a key onto an Object.");
+    }
     size = sw->depth * sw->out.indent + 3;
     if (sw->out.end - sw->out.cur <= (long)size) {
 	grow(&sw->out, size);
@@ -2081,9 +2175,34 @@ oj_str_writer_push_object(StrWriter sw, const char *key) {
     if (0 < sw->depth) {
 	fill_indent(&sw->out, sw->depth);
     }
-    if (0 != key) {
-	dump_cstr(key, strlen(key), 0, 0, &sw->out);
-	*sw->out.cur++ = ':';
+    dump_cstr(key, strlen(key), 0, 0, &sw->out);
+    *sw->out.cur++ = ':';
+    sw->keyWritten = 1;
+}
+
+void
+oj_str_writer_push_object(StrWriter sw, const char *key) {
+    if (sw->keyWritten) {
+	sw->keyWritten = 0;
+	if (sw->out.end - sw->out.cur <= 1) {
+	    grow(&sw->out, 1);
+	}
+    } else {
+	long	size;
+
+	key_check(sw, key);
+	size = sw->depth * sw->out.indent + 3;
+	if (sw->out.end - sw->out.cur <= (long)size) {
+	    grow(&sw->out, size);
+	}
+	maybe_comma(sw);
+	if (0 < sw->depth) {
+	    fill_indent(&sw->out, sw->depth);
+	}
+	if (0 != key) {
+	    dump_cstr(key, strlen(key), 0, 0, &sw->out);
+	    *sw->out.cur++ = ':';
+	}
     }
     *sw->out.cur++ = '{';
     push_type(sw, ObjectNew);
@@ -2091,20 +2210,27 @@ oj_str_writer_push_object(StrWriter sw, const char *key) {
 
 void
 oj_str_writer_push_array(StrWriter sw, const char *key) {
-    long	size;
+    if (sw->keyWritten) {
+	sw->keyWritten = 0;
+	if (sw->out.end - sw->out.cur <= 1) {
+	    grow(&sw->out, 1);
+	}
+    } else {
+	long	size;
 
-    key_check(sw, key);
-    size = sw->depth * sw->out.indent + 3;
-    if (sw->out.end - sw->out.cur <= size) {
-	grow(&sw->out, size);
-    }
-    maybe_comma(sw);
-    if (0 < sw->depth) {
-	fill_indent(&sw->out, sw->depth);
-    }
-    if (0 != key) {
-	dump_cstr(key, strlen(key), 0, 0, &sw->out);
-	*sw->out.cur++ = ':';
+	key_check(sw, key);
+	size = sw->depth * sw->out.indent + 3;
+	if (sw->out.end - sw->out.cur <= size) {
+	    grow(&sw->out, size);
+	}
+	maybe_comma(sw);
+	if (0 < sw->depth) {
+	    fill_indent(&sw->out, sw->depth);
+	}
+	if (0 != key) {
+	    dump_cstr(key, strlen(key), 0, 0, &sw->out);
+	    *sw->out.cur++ = ':';
+	}
     }
     *sw->out.cur++ = '[';
     push_type(sw, ArrayNew);
@@ -2112,40 +2238,48 @@ oj_str_writer_push_array(StrWriter sw, const char *key) {
 
 void
 oj_str_writer_push_value(StrWriter sw, VALUE val, const char *key) {
-    long	size;
+    if (sw->keyWritten) {
+	sw->keyWritten = 0;
+    } else {
+	long	size;
 
-    key_check(sw, key);
-    size = sw->depth * sw->out.indent + 3;
-    if (sw->out.end - sw->out.cur <= size) {
-	grow(&sw->out, size);
-    }
-    maybe_comma(sw);
-    if (0 < sw->depth) {
-	fill_indent(&sw->out, sw->depth);
-    }
-    if (0 != key) {
-	dump_cstr(key, strlen(key), 0, 0, &sw->out);
-	*sw->out.cur++ = ':';
+	key_check(sw, key);
+	size = sw->depth * sw->out.indent + 3;
+	if (sw->out.end - sw->out.cur <= size) {
+	    grow(&sw->out, size);
+	}
+	maybe_comma(sw);
+	if (0 < sw->depth) {
+	    fill_indent(&sw->out, sw->depth);
+	}
+	if (0 != key) {
+	    dump_cstr(key, strlen(key), 0, 0, &sw->out);
+	    *sw->out.cur++ = ':';
+	}
     }
     dump_val(val, sw->depth, &sw->out);
 }
 
 void
 oj_str_writer_push_json(StrWriter sw, const char *json, const char *key) {
-    long	size;
+    if (sw->keyWritten) {
+	sw->keyWritten = 0;
+    } else {
+	long	size;
 
-    key_check(sw, key);
-    size = sw->depth * sw->out.indent + 3;
-    if (sw->out.end - sw->out.cur <= size) {
-	grow(&sw->out, size);
-    }
-    maybe_comma(sw);
-    if (0 < sw->depth) {
-	fill_indent(&sw->out, sw->depth);
-    }
-    if (0 != key) {
-	dump_cstr(key, strlen(key), 0, 0, &sw->out);
-	*sw->out.cur++ = ':';
+	key_check(sw, key);
+	size = sw->depth * sw->out.indent + 3;
+	if (sw->out.end - sw->out.cur <= size) {
+	    grow(&sw->out, size);
+	}
+	maybe_comma(sw);
+	if (0 < sw->depth) {
+	    fill_indent(&sw->out, sw->depth);
+	}
+	if (0 != key) {
+	    dump_cstr(key, strlen(key), 0, 0, &sw->out);
+	    *sw->out.cur++ = ':';
+	}
     }
     dump_raw(json, strlen(json), &sw->out);
 }
@@ -2155,6 +2289,10 @@ oj_str_writer_pop(StrWriter sw) {
     long	size;
     DumpType	type = sw->types[sw->depth];
 
+    if (sw->keyWritten) {
+	sw->keyWritten = 0;
+	rb_raise(rb_eStandardError, "Can not pop after writing a key but no value.");
+    }
     sw->depth--;
     if (0 > sw->depth) {
 	rb_raise(rb_eStandardError, "Can not pop with no open array or object.");
