@@ -191,8 +191,10 @@ hat_value(ParseInfo pi, Val parent, const char *key, size_t klen, volatile VALUE
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "Invalid struct data");
 		return 1;
 	    }
+
 	    // If struct is not defined then we let this fail and raise an exception.
-	    sc = rb_const_get(oj_struct_class, rb_to_id(*RARRAY_PTR(value)));
+	    sc = oj_name2struct(pi, *RARRAY_PTR(value));
+
             // Create a properly initialized struct instance without calling the initialize method.
             parent->val = rb_obj_alloc(sc);
             // If the JSON array has more entries than the struct class allows, we record an error.
@@ -328,6 +330,13 @@ hash_set_cstr(ParseInfo pi, const char *key, size_t klen, const char *str, size_
     case T_HASH:
 	rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), str_to_value(pi, str, len, orig));
 	break;
+    case T_STRING:
+	if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
+	    rb_funcall(parent->val, oj_replace_id, 1, str_to_value(pi, str, len, orig));
+	} else {
+	    set_obj_ivar(parent, key, klen, str_to_value(pi, str, len, orig));
+	}
+	break;
     case T_OBJECT:
 	set_obj_ivar(parent, key, klen, str_to_value(pi, str, len, orig));
 	break;
@@ -411,19 +420,36 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
 	}
 	break;
     case T_HASH:
-	if (3 <= klen && '#' == key[1] && T_ARRAY == rb_type(value)) {
-	    long	len = RARRAY_LEN(value);
-	    VALUE	*a = RARRAY_PTR(value);
-	
-	    if (2 != len) {
-		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid hash pair");
-		return;
+
+	if (rb_cHash != rb_obj_class(parent->val)) {
+	    if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
+		rb_funcall(parent->val, oj_replace_id, 1, value);
+	    } else {
+		set_obj_ivar(parent, key, klen, value);
 	    }
-	    rb_hash_aset(parent->val, *a, a[1]);
 	} else {
-	    rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), value);
+	    if (3 <= klen && '#' == key[1] && T_ARRAY == rb_type(value)) {
+		long	len = RARRAY_LEN(value);
+		VALUE	*a = RARRAY_PTR(value);
+	
+		if (2 != len) {
+		    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid hash pair");
+		    return;
+		}
+		rb_hash_aset(parent->val, *a, a[1]);
+	    } else {
+		rb_hash_aset(parent->val, hash_key(pi, key, klen, parent->k1), value);
+	    }
 	}
 	break;
+    case T_ARRAY:
+	if (4 == klen && 's' == *key && 'e' == key[1] && 'l' == key[2] && 'f' == key[3]) {
+	    rb_funcall(parent->val, oj_replace_id, 1, value);
+	} else {
+	    set_obj_ivar(parent, key, klen, value);
+	}
+	break;
+    case T_STRING: // for subclassed strings
     case T_OBJECT:
 	set_obj_ivar(parent, key, klen, value);
 	break;
@@ -448,7 +474,6 @@ hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
     }
 }
 
-
 static VALUE
 start_hash(ParseInfo pi) {
     return Qnil;
@@ -497,21 +522,30 @@ add_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
     pi->stack.head->val = str_to_value(pi, str, len, orig);
 }
 
+void
+oj_set_object_callbacks(ParseInfo pi) {
+    oj_set_strict_callbacks(pi);
+    pi->end_hash = end_hash;
+    pi->start_hash = start_hash;
+    pi->hash_set_cstr = hash_set_cstr;
+    pi->hash_set_num = hash_set_num;
+    pi->hash_set_value = hash_set_value;
+    pi->add_cstr = add_cstr;
+    pi->array_append_cstr = array_append_cstr;
+}
+
 VALUE
 oj_object_parse(int argc, VALUE *argv, VALUE self) {
     struct _ParseInfo	pi;
 
     pi.options = oj_default_options;
-    oj_set_strict_callbacks(&pi);
-    pi.end_hash = end_hash;
-    pi.start_hash = start_hash;
-    pi.hash_set_cstr = hash_set_cstr;
-    pi.hash_set_num = hash_set_num;
-    pi.hash_set_value = hash_set_value;
-    pi.add_cstr = add_cstr;
-    pi.array_append_cstr = array_append_cstr;
+    oj_set_object_callbacks(&pi);
 
-    return oj_pi_parse(argc, argv, &pi, 0, 0);
+    if (T_STRING == rb_type(*argv)) {
+	return oj_pi_parse(argc, argv, &pi, 0, 0, 1);
+    } else {
+	return oj_pi_sparse(argc, argv, &pi, 0);
+    }
 }
 
 VALUE
@@ -528,5 +562,5 @@ oj_object_parse_cstr(int argc, VALUE *argv, char *json, size_t len) {
     pi.add_cstr = add_cstr;
     pi.array_append_cstr = array_append_cstr;
 
-    return oj_pi_parse(argc, argv, &pi, json, len);
+    return oj_pi_parse(argc, argv, &pi, json, len, 1);
 }
