@@ -35,6 +35,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include "oj.h"
 #include "parse.h"
@@ -67,6 +68,8 @@ ID	oj_json_create_id;
 ID	oj_length_id;
 ID	oj_new_id;
 ID	oj_read_id;
+ID	oj_readpartial_id;
+ID	oj_replace_id;
 ID	oj_string_id;
 ID	oj_to_hash_id;
 ID	oj_to_json_id;
@@ -110,6 +113,7 @@ static VALUE	float_sym;
 static VALUE	indent_sym;
 static VALUE	json_sym;
 static VALUE	mode_sym;
+static VALUE	nilnil_sym;
 static VALUE	null_sym;
 static VALUE	object_sym;
 static VALUE	ruby_sym;
@@ -118,6 +122,7 @@ static VALUE	strict_sym;
 static VALUE	symbol_keys_sym;
 static VALUE	time_format_sym;
 static VALUE	unix_sym;
+static VALUE	use_to_json_sym;
 static VALUE	xmlschema_sym;
 static VALUE	xss_safe_sym;
 
@@ -154,6 +159,8 @@ struct _Options	oj_default_options = {
     UnixTime,		// time_format
     Yes,		// bigdec_as_num
     AutoDec,		// bigdec_load
+    No,			// to_json
+    No,			// nilnil
     json_class,		// create_id
     10,			// create_id_len
     9,			// sec_prec
@@ -178,6 +185,8 @@ static VALUE	define_mimic_json(int argc, VALUE *argv, VALUE self);
  * - bigdecimal_load: [:bigdecimal|:float|:auto] load decimals as BigDecimal instead of as a Float. :auto pick the most precise for the number of digits.
  * - create_id: [String|nil] create id for json compatible object encoding, default is 'json_create'
  * - second_precision: [Fixnum|nil] number of digits after the decimal when dumping the seconds portion of time
+ * - use_to_json: [true|false|nil] call to_json() methods on dump, default is false
+ * - nilnil: [true|false|nil] if true a nil input to load will return nil and not raise an Exception
  * - allow_gc: [true|false|nil] allow or prohibit GC during parsing, default is true (allow)
  * @return [Hash] all current option settings.
  */
@@ -192,6 +201,8 @@ get_def_opts(VALUE self) {
     rb_hash_aset(opts, auto_define_sym, (Yes == oj_default_options.auto_define) ? Qtrue : ((No == oj_default_options.auto_define) ? Qfalse : Qnil));
     rb_hash_aset(opts, symbol_keys_sym, (Yes == oj_default_options.sym_key) ? Qtrue : ((No == oj_default_options.sym_key) ? Qfalse : Qnil));
     rb_hash_aset(opts, bigdecimal_as_decimal_sym, (Yes == oj_default_options.bigdec_as_num) ? Qtrue : ((No == oj_default_options.bigdec_as_num) ? Qfalse : Qnil));
+    rb_hash_aset(opts, use_to_json_sym, (Yes == oj_default_options.to_json) ? Qtrue : ((No == oj_default_options.to_json) ? Qfalse : Qnil));
+    rb_hash_aset(opts, nilnil_sym, (Yes == oj_default_options.nilnil) ? Qtrue : ((No == oj_default_options.nilnil) ? Qfalse : Qnil));
     rb_hash_aset(opts, allow_gc_sym, (Yes == oj_default_options.allow_gc) ? Qtrue : ((No == oj_default_options.allow_gc) ? Qfalse : Qnil));
     switch (oj_default_options.mode) {
     case StrictMode:	rb_hash_aset(opts, mode_sym, strict_sym);	break;
@@ -251,6 +262,8 @@ get_def_opts(VALUE self) {
  *        :ruby Time.to_s formatted String
  * @param [String|nil] :create_id create id for json compatible object encoding
  * @param [Fixnum|nil] :second_precision number of digits after the decimal when dumping the seconds portion of time
+ * @param [true|false|nil] :use_to_json call to_json() methods on dump, default is false
+ * @param [true|false|nil] :nilnil if true a nil input to load will return nil and not raise an Exception
  * @param [true|false|nil] :allow_gc allow or prohibit GC during parsing, default is true (allow)
  * @return [nil]
  */
@@ -262,6 +275,8 @@ set_def_opts(VALUE self, VALUE opts) {
 	{ symbol_keys_sym, &oj_default_options.sym_key },
 	{ class_cache_sym, &oj_default_options.class_cache },
 	{ bigdecimal_as_decimal_sym, &oj_default_options.bigdec_as_num },
+	{ use_to_json_sym, &oj_default_options.to_json },
+	{ nilnil_sym, &oj_default_options.nilnil },
 	{ allow_gc_sym, &oj_default_options.allow_gc },
 	{ Qnil, 0 }
     };
@@ -392,6 +407,8 @@ oj_parse_options(VALUE ropts, Options copts) {
 	{ symbol_keys_sym, &copts->sym_key },
 	{ class_cache_sym, &copts->class_cache },
 	{ bigdecimal_as_decimal_sym, &copts->bigdec_as_num },
+	{ use_to_json_sym, &copts->to_json },
+	{ nilnil_sym, &copts->nilnil },
 	{ allow_gc_sym, &copts->allow_gc },
 	{ Qnil, 0 }
     };
@@ -510,7 +527,7 @@ oj_parse_options(VALUE ropts, Options copts) {
 	    copts->escape_mode = JSONEsc;
 	}
     }
- }
+}
 
 /* Document-method: strict_load
  *	call-seq: strict_load(json, options) => Hash, Array, String, Fixnum, Float, true, false, or nil
@@ -519,9 +536,13 @@ oj_parse_options(VALUE ropts, Options copts) {
  * true, false, or nil. It parses using a mode that is strict in that it maps
  * each primitive JSON type to a similar Ruby type. The :create_id is not
  * honored in this mode. Note that a Ruby Hash is used to represent the JSON
- * Object type. These two are not the same since teh JSON Object type can have
+ * Object type. These two are not the same since the JSON Object type can have
  * repeating entries with the same key and Ruby Hash can not.
  *
+ * When used with a document that has multiple JSON elements the block, if
+ * any, will be yielded to. If no block then the last element read will be
+ * returned.
+ *
  * Raises an exception if the JSON is malformed or the classes specified are not
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
@@ -539,6 +560,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * on the :create_id value. It is not compatible in every way to every other
  * parser though as each parser has it's own variations.
  *
+ * When used with a document that has multiple JSON elements the block, if
+ * any, will be yielded to. If no block then the last element read will be
+ * returned.
+ *
  * Raises an exception if the JSON is malformed or the classes specified are not
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
@@ -556,6 +581,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * or dumped Object. The :auto_define and :circular options have meaning with
  * this parsing mode.
  *
+ * When used with a document that has multiple JSON elements the block, if
+ * any, will be yielded to. If no block then the last element read will be
+ * returned.
+ *
  * Raises an exception if the JSON is malformed or the classes specified are not
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
@@ -588,6 +617,14 @@ oj_parse_options(VALUE ropts, Options copts) {
  * specified are not valid. If the string input is not a valid JSON document (an
  * empty string is not a valid JSON document) an exception is raised.
  *
+ * When used with a document that has multiple JSON elements the block, if
+ * any, will be yielded to. If no block then the last element read will be
+ * returned.
+ *
+ * This parser operates on string and will attempt to load files into memory if
+ * a file object is passed as the first argument. A stream input will be parsed
+ * using a stream parser but others use the slightly faster string parser.
+ *
  * @param [String|IO] json JSON String or an Object that responds to read()
  * @param [Hash] options load options (same as default_options)
  */
@@ -638,40 +675,31 @@ load(int argc, VALUE *argv, VALUE self) {
  * specified are not valid. If the string input is not a valid JSON document (an
  * empty string is not a valid JSON document) an exception is raised.
  *
+ * When used with a document that has multiple JSON elements the block, if
+ * any, will be yielded to. If no block then the last element read will be
+ * returned.
+ *
  * If the input file is not a valid JSON document (an empty file is not a valid
  * JSON document) an exception is raised.
  *
+ * This is a stream based parser which allows a large or huge file to be loaded
+ * without pulling the whole file into memory.
+ *
  * @param [String] path path to a file containing a JSON document
  * @param [Hash] options load options (same as default_options)
  */
 static VALUE
 load_file(int argc, VALUE *argv, VALUE self) {
     char		*path;
-    char		*json;
-    FILE		*f;
-    unsigned long	len;
+    int			fd;
     Mode		mode = oj_default_options.mode;
-
-    Check_Type(*argv, T_STRING);
-    path = StringValuePtr(*argv);
-    if (0 == (f = fopen(path, "r"))) {
-	rb_raise(rb_eIOError, "%s", strerror(errno));
-    }
-    fseek(f, 0, SEEK_END);
-    len = ftell(f);
-    json = ALLOC_N(char, len + 1);
-    fseek(f, 0, SEEK_SET);
-    if (len != fread(json, 1, len, f)) {
-	xfree(json);
-	fclose(f);
-	rb_raise(rb_const_get_at(Oj, rb_intern("LoadError")), "Failed to read %ld bytes from %s.", len, path);
-    }
-    fclose(f);
-    json[len] = '\0';
+    struct _ParseInfo	pi;
 
     if (1 > argc) {
 	rb_raise(rb_eArgError, "Wrong number of arguments to load().");
     }
+    Check_Type(*argv, T_STRING);
+    pi.options = oj_default_options;
     if (2 <= argc) {
 	VALUE	ropts = argv[1];
 	VALUE	v;
@@ -690,18 +718,25 @@ load_file(int argc, VALUE *argv, VALUE self) {
 	    }
 	}
     }
-    // The json string is freed in the parser when it is finished with it.
+    path = StringValuePtr(*argv);
+    if (0 == (fd = open(path, O_RDONLY))) {
+	rb_raise(rb_eIOError, "%s", strerror(errno));
+    }
     switch (mode) {
     case StrictMode:
-	return oj_strict_parse_cstr(argc, argv, json, len);
+	oj_set_strict_callbacks(&pi);
+	return oj_pi_sparse(argc, argv, &pi, fd);
     case NullMode:
     case CompatMode:
-	return oj_compat_parse_cstr(argc, argv, json, len);
+	oj_set_compat_callbacks(&pi);
+	return oj_pi_sparse(argc, argv, &pi, fd);
     case ObjectMode:
     default:
 	break;
     }
-    return oj_object_parse_cstr(argc, argv, json, len);
+    oj_set_object_callbacks(&pi);
+
+    return oj_pi_sparse(argc, argv, &pi, fd);
 }
 
 /* call-seq: safe_load(doc)
@@ -725,7 +760,7 @@ safe_load(VALUE self, VALUE doc) {
     oj_set_strict_callbacks(&pi);
     *args = doc;
 
-    return oj_pi_parse(1, args, &pi, 0, 0);
+    return oj_pi_parse(1, args, &pi, 0, 0, 1);
 }
 
 /* call-seq: saj_parse(handler, io)
@@ -827,6 +862,37 @@ to_stream(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
+/* call-seq: register_odd(clas, create_object, create_method, *members)
+ *
+ * Registers a class as special. This is useful for working around subclasses of
+ * primitive types as is done with ActiveSupport classes. The use of this
+ * function should be limited to just classes that can not be handled in the
+ * normal way. It is not intended as a hook for changing the output of all
+ * classes as it is not optimized for large numbers of classes.
+ *
+ * @param [Class] clas Class to me made special
+ * @param [Object] create_object object to call the create method on
+ * @param [Symbol] create_method method on the clas that will create a new
+ *                 instance of the clas when given all the member values in the
+ *                 order specified.
+ * @param [Symbol|String] members methods used to get the member values from
+ *                        instances of the clas
+ */
+static VALUE
+register_odd(int argc, VALUE *argv, VALUE self) {
+    if (3 > argc) {
+	rb_raise(rb_eArgError, "incorrect number of arguments.");
+    }
+    Check_Type(argv[0], T_CLASS);
+    Check_Type(argv[2], T_SYMBOL);
+    if (MAX_ODD_ARGS < argc - 2) {
+	rb_raise(rb_eArgError, "too many members.");
+    }
+    oj_reg_odd(argv[0], argv[1], argv[2], argc - 3, argv + 3);
+
+    return Qnil;
+}
+
 static void
 str_writer_free(void *ptr) {
     StrWriter	sw;
@@ -857,6 +923,8 @@ str_writer_init(StrWriter sw) {
     sw->types = ALLOC_N(char, 256);
     sw->types_end = sw->types + 256;
     *sw->types = '\0';
+    sw->keyWritten = 0;
+
     sw->out.buf = ALLOC_N(char, 4096);
     sw->out.end = sw->out.buf + 4086;
     sw->out.allocated = 1;
@@ -887,6 +955,23 @@ str_writer_new(int argc, VALUE *argv, VALUE self) {
     return Data_Wrap_Struct(oj_string_writer_class, 0, str_writer_free, sw);
 }
 
+/* call-seq: push_key(key)
+ *
+ * Pushes a key onto the JSON document. The key will be used for the next push
+ * if currently in a JSON object and ignored otherwise. If a key is provided on
+ * the next push then that new key will be ignored.
+ * @param [String] key the key pending for the next push
+ */
+static VALUE
+str_writer_push_key(VALUE self, VALUE key) {
+    StrWriter	sw = (StrWriter)DATA_PTR(self);
+
+    rb_check_type(key, T_STRING);
+    oj_str_writer_push_key(sw, StringValuePtr(key));
+
+    return Qnil;
+}
+
 /* call-seq: push_object(key=nil)
  *
  * Pushes an object onto the JSON document. Future pushes will be to this object
@@ -1043,6 +1128,7 @@ str_writer_reset(VALUE self) {
 
     sw->depth = 0;
     *sw->types = '\0';
+    sw->keyWritten = 0;
     sw->out.cur = sw->out.buf;
     *sw->out.cur = '\0';
 
@@ -1127,9 +1213,10 @@ stream_writer_new(int argc, VALUE *argv, VALUE self) {
     if (oj_stringio_class == clas) {
 	type = STRING_IO;
 #if !IS_WINDOWS
-    } else if (rb_respond_to(stream, oj_fileno_id) && Qnil != (s = rb_funcall(stream, oj_fileno_id, 0))) {
+    } else if (rb_respond_to(stream, oj_fileno_id) &&
+	       Qnil != (s = rb_funcall(stream, oj_fileno_id, 0)) &&
+	       0 != (fd = FIX2INT(s))) {
 	type = FILE_IO;
-	fd = FIX2INT(s);
 #endif
     } else if (rb_respond_to(stream, oj_write_id)) {
 	type = STREAM_IO;
@@ -1148,6 +1235,24 @@ stream_writer_new(int argc, VALUE *argv, VALUE self) {
     return Data_Wrap_Struct(oj_stream_writer_class, 0, stream_writer_free, sw);
 }
 
+/* call-seq: push_key(key)
+ *
+ * Pushes a key onto the JSON document. The key will be used for the next push
+ * if currently in a JSON object and ignored otherwise. If a key is provided on
+ * the next push then that new key will be ignored.
+ * @param [String] key the key pending for the next push
+ */
+static VALUE
+stream_writer_push_key(VALUE self, VALUE key) {
+    StreamWriter	sw = (StreamWriter)DATA_PTR(self);
+
+    rb_check_type(key, T_STRING);
+    stream_writer_reset_buf(sw);
+    oj_str_writer_push_key(&sw->sw, StringValuePtr(key));
+    stream_writer_write(sw);
+    return Qnil;
+}
+
 /* call-seq: push_object(key=nil)
  *
  * Pushes an object onto the JSON document. Future pushes will be to this object
@@ -1347,11 +1452,11 @@ mimic_walk(VALUE key, VALUE obj, VALUE proc) {
 	break;
     case T_ARRAY:
 	{
-	    VALUE	*np = RARRAY_PTR(obj);
 	    size_t	cnt = RARRAY_LEN(obj);
-	
-	    for (; 0 < cnt; cnt--, np++) {
-		mimic_walk(Qnil, *np, proc);
+	    size_t	i;
+
+	    for (i = 0; i < cnt; i++) {
+		mimic_walk(Qnil, rb_ary_entry(obj, i), proc);
 	    }
 	    break;
 	}
@@ -1377,9 +1482,14 @@ mimic_walk(VALUE key, VALUE obj, VALUE proc) {
 
 static VALUE
 mimic_load(int argc, VALUE *argv, VALUE self) {
-    VALUE	obj = load(1, argv, self);
-    VALUE	p = Qnil;
+    struct _ParseInfo	pi;
+    VALUE		obj;
+    VALUE		p = Qnil;
 
+    pi.options = oj_default_options;
+    oj_set_compat_callbacks(&pi);
+
+    obj = oj_pi_parse(argc, argv, &pi, 0, 0, 0);
     if (2 <= argc) {
 	p = argv[1];
     }
@@ -1533,7 +1643,7 @@ mimic_parse(int argc, VALUE *argv, VALUE self) {
     }
     *args = *argv;
 
-    return oj_pi_parse(1, args, &pi, 0, 0);
+    return oj_pi_parse(1, args, &pi, 0, 0, 0);
 }
 
 static VALUE
@@ -1598,7 +1708,7 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
     }
     if (rb_const_defined_at(mimic, rb_intern("Ext"))) {
 	ext = rb_const_get_at(mimic, rb_intern("Ext"));
-    } else {
+     } else {
 	ext = rb_define_module_under(mimic, "Ext");
     }
     if (!rb_const_defined_at(ext, rb_intern("Parser"))) {
@@ -1651,8 +1761,12 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
     space_sym = ID2SYM(rb_intern("space"));				rb_gc_register_address(&space_sym);
     symbolize_names_sym = ID2SYM(rb_intern("symbolize_names"));		rb_gc_register_address(&symbolize_names_sym);
 
+    if (!rb_const_defined_at(mimic, rb_intern("ParserError"))) {
+	rb_define_const(mimic, "ParserError", oj_parse_error_class);
+    }
     oj_default_options.mode = CompatMode;
     oj_default_options.escape_mode = ASCIIEsc;
+    oj_default_options.nilnil = Yes;
 
     return mimic;
 }
@@ -1699,6 +1813,7 @@ void Init_oj() {
 
     oj_string_writer_class = rb_define_class_under(Oj, "StringWriter", rb_cObject);
     rb_define_module_function(oj_string_writer_class, "new", str_writer_new, -1);
+    rb_define_method(oj_string_writer_class, "push_key", str_writer_push_key, 1);
     rb_define_method(oj_string_writer_class, "push_object", str_writer_push_object, -1);
     rb_define_method(oj_string_writer_class, "push_array", str_writer_push_array, -1);
     rb_define_method(oj_string_writer_class, "push_value", str_writer_push_value, -1);
@@ -1710,6 +1825,7 @@ void Init_oj() {
 
     oj_stream_writer_class = rb_define_class_under(Oj, "StreamWriter", rb_cObject);
     rb_define_module_function(oj_stream_writer_class, "new", stream_writer_new, -1);
+    rb_define_method(oj_stream_writer_class, "push_key", stream_writer_push_key, 1);
     rb_define_method(oj_stream_writer_class, "push_object", stream_writer_push_object, -1);
     rb_define_method(oj_stream_writer_class, "push_array", stream_writer_push_array, -1);
     rb_define_method(oj_stream_writer_class, "push_value", stream_writer_push_value, -1);
@@ -1749,6 +1865,7 @@ void Init_oj() {
     rb_define_module_function(Oj, "dump", dump, -1);
     rb_define_module_function(Oj, "to_file", to_file, -1);
     rb_define_module_function(Oj, "to_stream", to_stream, -1);
+    rb_define_module_function(Oj, "register_odd", register_odd, -1);
 
     rb_define_module_function(Oj, "saj_parse", oj_saj_parse, -1);
     rb_define_module_function(Oj, "sc_parse", oj_sc_parse, -1);
@@ -1769,6 +1886,8 @@ void Init_oj() {
     oj_length_id = rb_intern("length");
     oj_new_id = rb_intern("new");
     oj_read_id = rb_intern("read");
+    oj_readpartial_id = rb_intern("readpartial");
+    oj_replace_id = rb_intern("replace");
     oj_string_id = rb_intern("string");
     oj_to_hash_id = rb_intern("to_hash");
     oj_to_json_id = rb_intern("to_json");
@@ -1781,6 +1900,12 @@ void Init_oj() {
     oj_utc_offset_id = rb_intern("utc_offset");
     oj_write_id = rb_intern("write");
 
+    rb_require("oj/bag");
+    rb_require("oj/error");
+    rb_require("oj/mimic");
+    rb_require("oj/saj");
+    rb_require("oj/schandler");
+
     oj_bag_class = rb_const_get_at(Oj, rb_intern("Bag"));
     oj_bigdecimal_class = rb_const_get(rb_cObject, rb_intern("BigDecimal"));
     oj_date_class = rb_const_get(rb_cObject, rb_intern("Date"));
@@ -1799,6 +1924,7 @@ void Init_oj() {
     bigdecimal_load_sym = ID2SYM(rb_intern("bigdecimal_load"));rb_gc_register_address(&bigdecimal_load_sym);
     bigdecimal_sym = ID2SYM(rb_intern("bigdecimal"));	rb_gc_register_address(&bigdecimal_sym);
     circular_sym = ID2SYM(rb_intern("circular"));	rb_gc_register_address(&circular_sym);
+    nilnil_sym = ID2SYM(rb_intern("nilnil"));		rb_gc_register_address(&nilnil_sym);
     class_cache_sym = ID2SYM(rb_intern("class_cache"));	rb_gc_register_address(&class_cache_sym);
     compat_sym = ID2SYM(rb_intern("compat"));		rb_gc_register_address(&compat_sym);
     create_id_sym = ID2SYM(rb_intern("create_id"));	rb_gc_register_address(&create_id_sym);
@@ -1815,6 +1941,7 @@ void Init_oj() {
     symbol_keys_sym = ID2SYM(rb_intern("symbol_keys"));	rb_gc_register_address(&symbol_keys_sym);
     time_format_sym = ID2SYM(rb_intern("time_format"));	rb_gc_register_address(&time_format_sym);
     unix_sym = ID2SYM(rb_intern("unix"));		rb_gc_register_address(&unix_sym);
+    use_to_json_sym = ID2SYM(rb_intern("use_to_json"));	rb_gc_register_address(&use_to_json_sym);
     xmlschema_sym = ID2SYM(rb_intern("xmlschema"));	rb_gc_register_address(&xmlschema_sym);
     xss_safe_sym = ID2SYM(rb_intern("xss_safe"));	rb_gc_register_address(&xss_safe_sym);
 
