@@ -640,6 +640,20 @@ oj_parse2(ParseInfo pi) {
 	if (err_has(&pi->err)) {
 	    return;
 	}
+	if (Qundef != pi->proc && stack_empty(&pi->stack)) {
+	    if (Qnil == pi->proc) {
+		rb_yield(stack_head_val(&pi->stack));
+	    } else {
+#if HAS_PROC_WITH_BLOCK
+		VALUE	args[1];
+
+		*args = stack_head_val(&pi->stack);
+		rb_proc_call_with_block(pi->proc, 1, args, Qnil);
+#else
+		rb_raise(rb_eNotImpError, "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
+#endif
+	    }
+	}
     }
 }
 
@@ -708,7 +722,11 @@ oj_set_error_at(ParseInfo pi, VALUE err_clas, const char* file, int line, const
     vsnprintf(msg, sizeof(msg) - 1, format, ap);
     va_end(ap);
     pi->err.clas = err_clas;
-    _oj_err_set_with_location(&pi->err, err_clas, msg, pi->json, pi->cur - 1, file, line);
+    if (0 == pi->json) {
+	oj_err_set(&pi->err, err_clas, "%s at line %d, column %d [%s:%d]", msg, pi->rd.line, pi->rd.col, file, line);
+    } else {
+	_oj_err_set_with_location(&pi->err, err_clas, msg, pi->json, pi->cur - 1, file, line);
+    }
 }
 
 static VALUE
@@ -719,7 +737,7 @@ protect_parse(VALUE pip) {
 }
 
 VALUE
-oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len) {
+oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yieldOk) {
     char		*buf = 0;
     volatile VALUE	input;
     volatile VALUE	wrapped_stack;
@@ -734,25 +752,36 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len) {
     if (2 == argc) {
 	oj_parse_options(argv[1], &pi->options);
     }
+    if (yieldOk && rb_block_given_p()) {
+	pi->proc = Qnil;
+    } else {
+	pi->proc = Qundef;
+    }
     pi->cbc = (void*)0;
     if (0 != json) {
 	pi->json = json;
 	pi->end = json + len;
 	free_json = 1;
-    } else if (rb_type(input) == T_STRING) {
+    } else if (T_STRING == rb_type(input)) {
 	pi->json = rb_string_value_cstr((VALUE*)&input);
 	pi->end = pi->json + RSTRING_LEN(input);
+    } else if (Qnil == input && Yes == pi->options.nilnil) {
+	return Qnil;
     } else {
 	VALUE		clas = rb_obj_class(input);
 	volatile VALUE	s;
+#if !IS_WINDOWS
+	int		fd;
+#endif
 
 	if (oj_stringio_class == clas) {
 	    s = rb_funcall2(input, oj_string_id, 0, 0);
 	    pi->json = rb_string_value_cstr((VALUE*)&s);
 	    pi->end = pi->json + RSTRING_LEN(s);
 #if !IS_WINDOWS
-	} else if (rb_respond_to(input, oj_fileno_id) && Qnil != (s = rb_funcall(input, oj_fileno_id, 0))) {
-	    int		fd = FIX2INT(s);
+	} else if (rb_respond_to(input, oj_fileno_id) &&
+		   Qnil != (s = rb_funcall(input, oj_fileno_id, 0)) &&
+		   0 != (fd = FIX2INT(s))) {
 	    ssize_t	cnt;
 	    size_t	len = lseek(fd, 0, SEEK_END);
 
@@ -773,9 +802,8 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len) {
 	    }
 #endif
 	} else if (rb_respond_to(input, oj_read_id)) {
-	    s = rb_funcall2(input, oj_read_id, 0, 0);
-	    pi->json = rb_string_value_cstr((VALUE*)&s);
-	    pi->end = pi->json + RSTRING_LEN(s);
+	    // use stream parser instead
+	    return oj_pi_sparse(argc, argv, pi, 0);
 	} else {
 	    rb_raise(rb_eArgError, "strict_parse() expected a String or IO Object.");
 	}
@@ -799,6 +827,29 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len) {
     if (No == pi->options.allow_gc) {
 	rb_gc_enable();
     }
+    if (!err_has(&pi->err)) {
+	// If the stack is not empty then the JSON terminated early.
+	Val	v;
+
+	if (0 != (v = stack_peek(&pi->stack))) {
+	    switch (v->next) {
+	    case NEXT_ARRAY_NEW:
+	    case NEXT_ARRAY_ELEMENT:
+	    case NEXT_ARRAY_COMMA:
+		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "Array not terminated");
+		break;
+	    case NEXT_HASH_NEW:
+	    case NEXT_HASH_KEY:
+	    case NEXT_HASH_COLON:
+	    case NEXT_HASH_VALUE:
+	    case NEXT_HASH_COMMA:
+		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "Hash/Object not terminated");
+		break;
+	    default:
+		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not terminated");
+	    }
+	}
+    }
     // proceed with cleanup
     if (0 != pi->circ_array) {
 	oj_circ_array_free(pi->circ_array);
