@@ -32,11 +32,13 @@
 #define __OJ_PARSE_H__
 
 #include <stdarg.h>
+#include <stdio.h>
 
 #include "ruby.h"
 #include "oj.h"
 #include "val_stack.h"
 #include "circarray.h"
+#include "reader.h"
 
 typedef struct _NumInfo {
     int64_t	i;
@@ -54,15 +56,20 @@ typedef struct _NumInfo {
 } *NumInfo;
 
 typedef struct _ParseInfo {
+    // used for the string parser
     const char		*json;
     const char		*cur;
     const char		*end;
+    // used for the stream parser
+    struct _Reader	rd;
+
     struct _Err		err;
     struct _Options	options;
     void		*cbc;
     struct _ValStack	stack;
     CircArray		circ_array;
     int			expect_value;
+    VALUE		proc;
     VALUE		(*start_hash)(struct _ParseInfo *pi);
     void		(*end_hash)(struct _ParseInfo *pi);
     void		(*hash_set_cstr)(struct _ParseInfo *pi, const char *key, size_t klen, const char *str, size_t len, const char *orig);
@@ -82,10 +89,14 @@ typedef struct _ParseInfo {
 
 extern void	oj_parse2(ParseInfo pi);
 extern void	oj_set_error_at(ParseInfo pi, VALUE err_clas, const char* file, int line, const char *format, ...);
-extern VALUE	oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len);
+extern VALUE	oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yieldOk);
 extern VALUE	oj_num_as_value(NumInfo ni);
 
 extern void	oj_set_strict_callbacks(ParseInfo pi);
+extern void	oj_set_object_callbacks(ParseInfo pi);
 extern void	oj_set_compat_callbacks(ParseInfo pi);
 
+extern void	oj_sparse2(ParseInfo pi);
+extern VALUE	oj_pi_sparse(int argc, VALUE *argv, ParseInfo pi, int fd);
+
 #endif /* __OJ_PARSE_H__ */
