@@ -1,4 +1,4 @@
-/* sajkey.c
+/* saj.c
  * Copyright (c) 2012, Peter Ohler
  * All rights reserved.
  * 
@@ -120,23 +120,6 @@ next_non_white(ParseInfo pi) {
     }
 }
 
-inline static void
-next_white(ParseInfo pi) {
-    for (; 1; pi->s++) {
-	switch(*pi->s) {
-	case ' ':
-	case '\t':
-	case '\f':
-	case '\n':
-	case '\r':
-	case '\0':
-	    return;
-	default:
-	    break;
-	}
-    }
-}
-
 inline static void
 call_add_value(VALUE handler, VALUE value, const char *key) {
     volatile VALUE	k;
@@ -661,7 +644,7 @@ respond_to(VALUE obj, ID method) {
 }
 
 static void
-sajkey_parse(VALUE handler, char *json) {
+saj_parse(VALUE handler, char *json) {
     volatile VALUE	obj = Qnil;
     struct _ParseInfo	pi;
 
@@ -709,11 +692,11 @@ sajkey_parse(VALUE handler, char *json) {
     }
 }
 
-/* call-seq: sajkey_parse(handler, io)
+/* call-seq: saj_parse(handler, io)
  *
  * Parses an IO stream or file containing an JSON document. Raises an exception
  * if the JSON is malformed.
- * @param [Oj::SajKey] handler SajKey (responds to Oj::SajKey methods) like handler
+ * @param [Oj::Saj] handler Saj (responds to Oj::Saj methods) like handler
  * @param [IO|String] io IO Object to read from
  */
 VALUE
@@ -733,17 +716,19 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
     } else {
 	VALUE		clas = rb_obj_class(input);
 	volatile VALUE	s;
-
+#if !IS_WINDOWS
+	int		fd;
+#endif
+	
 	if (oj_stringio_class == clas) {
 	    s = rb_funcall2(input, oj_string_id, 0, 0);
 	    len = RSTRING_LEN(s) + 1;
 	    json = ALLOC_N(char, len);
 	    strcpy(json, rb_string_value_cstr((VALUE*)&s));
-#ifndef JRUBY_RUBY
 #if !IS_WINDOWS
-	    // JRuby gets confused with what is the real fileno.
-	} else if (rb_respond_to(input, oj_fileno_id) && Qnil != (s = rb_funcall(input, oj_fileno_id, 0))) {
-	    int		fd = FIX2INT(s);
+	} else if (rb_respond_to(input, oj_fileno_id) &&
+		   Qnil != (s = rb_funcall(input, oj_fileno_id, 0)) &&
+		   0 != (fd = FIX2INT(s))) {
 	    ssize_t	cnt;
 
 	    len = lseek(fd, 0, SEEK_END);
@@ -753,7 +738,6 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
 		rb_raise(rb_eIOError, "failed to read from IO Object.");
 	    }
 	    json[len] = '\0';
-#endif
 #endif
 	} else if (rb_respond_to(input, oj_read_id)) {
 	    s = rb_funcall2(input, oj_read_id, 0, 0);
@@ -764,7 +748,7 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
 	    rb_raise(rb_eArgError, "saj_parse() expected a String or IO Object.");
 	}
     }
-    sajkey_parse(*argv, json);
+    saj_parse(*argv, json);
     xfree(json);
 
     return Qnil;
