@@ -211,6 +211,11 @@ oj_sc_parse(int argc, VALUE *argv, VALUE self) {
     if (3 == argc) {
 	oj_parse_options(argv[2], &pi.options);
     }
+    if (rb_block_given_p()) {
+	pi.proc = Qnil;
+    } else {
+	pi.proc = Qundef;
+    }
     pi.cbc = (void*)handler;
 
     pi.start_hash = respond_to(handler, oj_hash_start_id) ? start_hash : noop_start;
@@ -255,15 +260,17 @@ oj_sc_parse(int argc, VALUE *argv, VALUE self) {
     } else {
 	VALUE		clas = rb_obj_class(input);
 	volatile VALUE	s;
-
+#if !IS_WINDOWS
+	int		fd;
+#endif
 	if (oj_stringio_class == clas) {
 	    s = rb_funcall2(input, oj_string_id, 0, 0);
 	    pi.json = StringValuePtr(input);
-#ifndef JRUBY_RUBY
 #if !IS_WINDOWS
 	    // JRuby gets confused with what is the real fileno.
-	} else if (rb_respond_to(input, oj_fileno_id) && Qnil != (s = rb_funcall(input, oj_fileno_id, 0))) {
-	    int		fd = FIX2INT(s);
+	} else if (rb_respond_to(input, oj_fileno_id) &&
+		   Qnil != (s = rb_funcall(input, oj_fileno_id, 0)) &&
+		   0 != (fd = FIX2INT(s))) {
 	    ssize_t	cnt;
 	    size_t	len = lseek(fd, 0, SEEK_END);
 
@@ -277,7 +284,6 @@ oj_sc_parse(int argc, VALUE *argv, VALUE self) {
 		rb_raise(rb_eIOError, "failed to read from IO Object.");
 	    }
 	    ((char*)pi.json)[len] = '\0';
-#endif
 #endif
 	} else if (rb_respond_to(input, oj_read_id)) {
 	    s = rb_funcall2(input, oj_read_id, 0, 0);
