@@ -12,13 +12,13 @@ $: << File.join(File.dirname(__FILE__), "../ext")
 
 require 'oj'
 
-reltypes={}
-Oj::Doc.open_file('foo.json') do |doc|
-  doc.each_child do |target|
-    puts "#{target.local_key} is #{target.local_key.class}"
-    target.each_leaf do |score|
-      reltype=score.local_key
-      reltypes[reltype] = (reltypes[reltype] || 0) + 1
-    end
+class Foo < String
+  def initialize(str, safe)
+    super(str)
+    @safe = safe
   end
 end
+
+f = Foo.new("z")
+
+puts f
