@@ -32,8 +32,8 @@ class Jeez
   end
   alias == eql?
   
-  def to_json(*a)
-    %{{"json_class":"#{self.class}","x":#{@x},"y":#{@y}}}
+  def as_json()
+    {"json_class" => self.class.to_s,"x" => @x,"y" => @y}
   end
 
   def self.json_create(h)
