@@ -31,8 +31,8 @@ class Jam
   end
   alias == eql?
 
-  def to_json()
-    %{{"json_class":"#{self.class}","x":#{@x},"y":#{@y}}}
+  def as_json()
+    {"json_class" => self.class.to_s,"x" => @x,"y" => @y}
   end
 
   def self.json_create(h)
@@ -80,19 +80,16 @@ class Mimic < ::Test::Unit::TestCase
     Oj.mimic_JSON # TBD
     children = []
     json = %{{"a":1,"b":[true,false]}}
-    if 'rubinius' == $ruby || 'jruby' == $ruby || '1.8.7' == RUBY_VERSION
+    if 'rubinius' == $ruby || '1.8.7' == RUBY_VERSION
       obj = JSON.load(json) {|x| children << x }
     else
       p = Proc.new {|x| children << x }
       obj = JSON.load(json, p)
     end
     assert_equal({ 'a' => 1, 'b' => [true, false]}, obj)
-    # JRuby 1.7.0 rb_yield() is broken and converts the [true, falser] array into true
-    unless 'jruby' == $ruby && '1.9.3' == RUBY_VERSION
-      assert([1, true, false, [true, false], { 'a' => 1, 'b' => [true, false]}] == children ||
-             [true, false, [true, false], 1, { 'a' => 1, 'b' => [true, false]}] == children,
-             "children don't match")
-    end
+    assert([1, true, false, [true, false], { 'a' => 1, 'b' => [true, false]}] == children ||
+           [true, false, [true, false], 1, { 'a' => 1, 'b' => [true, false]}] == children,
+           "children don't match")
   end
 
 # []
@@ -206,4 +203,17 @@ class Mimic < ::Test::Unit::TestCase
     end
   end
 
+  # exception check
+  def test_exception
+    json = %{["a":1]}
+    begin
+      obj = JSON.parse(json)
+      
+    rescue JSON::ParserError => je
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an JSON::ParserError")
+  end
+
 end # Mimic
