@@ -13,7 +13,6 @@ $: << File.join(File.dirname(__FILE__), "../ext")
 require 'test/unit'
 require 'stringio'
 require 'date'
-require 'bigdecimal'
 require 'oj'
 
 class OjWriter < ::Test::Unit::TestCase
@@ -62,6 +61,26 @@ class OjWriter < ::Test::Unit::TestCase
     assert_equal('{"a1":{},"a2":{"b":{}},"a3":{}}', w.to_s)
   end
 
+  def test_string_writer_nested_key
+    w = Oj::StringWriter.new(:indent => 0)
+    w.push_object()
+    w.push_key('a1')
+    w.push_object()
+    w.pop()
+    w.push_key('a2')
+    w.push_object('x')
+    w.push_object('b')
+    w.pop()
+    w.pop()
+    w.push_key('a3')
+    w.push_object()
+    w.push_key('a4')
+    w.push_value(37)
+    w.pop()
+    w.pop()
+    assert_equal('{"a1":{},"a2":{"b":{}},"a3":{"a4":37}}', w.to_s)
+  end
+
   def test_string_writer_value_array
     w = Oj::StringWriter.new(:indent => 2)
     w.push_array()
@@ -118,6 +137,33 @@ class OjWriter < ::Test::Unit::TestCase
   end
 
   def test_string_writer_pop_excess
+    w = Oj::StringWriter.new(:indent => 0)
+    begin
+      w.push_object() {
+        w.push_key('key')
+      }
+    rescue Exception
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
+  def test_string_writer_array_key
+    w = Oj::StringWriter.new(:indent => 0)
+    begin
+      w.push_array() {
+        w.push_key('key')
+        w.push_value(7)
+      }
+    rescue Exception
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
+  def test_string_writer_pop_with_key
     w = Oj::StringWriter.new(:indent => 0)
     begin
       w.pop()
@@ -224,4 +270,25 @@ class OjWriter < ::Test::Unit::TestCase
     assert_equal('{"a1":{},"a2":{"b":[7,true,"string"]},"a3":{}}', content)
   end
 
+  def test_stream_writer_nested_key_object
+    output = StringIO.open("", "w+")
+    w = Oj::StreamWriter.new(output, :indent => 0)
+    w.push_object()
+    w.push_key('a1')
+    w.push_object()
+    w.pop()
+    w.push_key('a2')
+    w.push_object('x')
+    w.push_object('b')
+    w.pop()
+    w.pop()
+    w.push_key('a3')
+    w.push_object()
+    w.push_key('a4')
+    w.push_value(37)
+    w.pop()
+    w.pop()
+    assert_equal('{"a1":{},"a2":{"b":{}},"a3":{"a4":37}}', output.string())
+  end
+
 end # OjWriter
