@@ -134,6 +134,8 @@ class Juice < ::Test::Unit::TestCase
                    :time_format=>:unix,
                    :bigdecimal_as_decimal=>true,
                    :bigdecimal_load=>:auto,
+                   :use_to_json=>false,
+                   :nilnil=>false,
                    :allow_gc=>true,
                    :create_id=>'json_class'}, opts)
   end
@@ -151,6 +153,8 @@ class Juice < ::Test::Unit::TestCase
       :time_format=>:unix,
       :bigdecimal_as_decimal=>true,
       :bigdecimal_load=>:auto,
+      :use_to_json=>true,
+      :nilnil=>false,
       :allow_gc=>true,
       :create_id=>'json_class'}
     o2 = {
@@ -165,6 +169,8 @@ class Juice < ::Test::Unit::TestCase
       :time_format=>:ruby,
       :bigdecimal_as_decimal=>false,
       :bigdecimal_load=>:bigdecimal,
+      :use_to_json=>false,
+      :nilnil=>true,
       :allow_gc=>false,
       :create_id=>nil}
     o3 = { :indent => 4 }
@@ -253,6 +259,15 @@ class Juice < ::Test::Unit::TestCase
     dump_and_load([[nil]], false)
     dump_and_load([[nil], 58], false)
   end
+  def test_array_not_closed
+    begin
+      Oj.load('[')
+    rescue Exception => e
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
 
   # rails encoding tests
   def test_does_not_escape_entities_by_default
@@ -314,10 +329,11 @@ class Juice < ::Test::Unit::TestCase
     t = Time.local(2012, 1, 5, 23, 58, 7)
     begin
       Oj.dump(t, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_time_null
     t = Time.local(2012, 1, 5, 23, 58, 7)
@@ -459,10 +475,11 @@ class Juice < ::Test::Unit::TestCase
   def test_class_strict
     begin
       Oj.dump(Juice, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_class_null
     json = Oj.dump(Juice, :mode => :null)
@@ -487,18 +504,20 @@ class Juice < ::Test::Unit::TestCase
   def test_non_str_hash_strict
     begin
       Oj.dump({ 1 => true, 0 => false }, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_non_str_hash_null
     begin
       Oj.dump({ 1 => true, 0 => false }, :mode => :null)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_non_str_hash_compat
     json = Oj.dump({ 1 => true, 0 => false }, :mode => :compat)
@@ -521,16 +540,26 @@ class Juice < ::Test::Unit::TestCase
     h = Oj.load(json)
     assert_equal({ 1 => true, 'nil' => nil, :sim => 4 }, h)
   end
+  def test_hash_not_closed
+    begin
+      Oj.load('{')
+    rescue Exception => e
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
 
   # Object with to_json()
   def test_json_object_strict
     obj = Jeez.new(true, 58)
     begin
       Oj.dump(obj, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_json_object_null
     obj = Jeez.new(true, 58)
@@ -538,12 +567,15 @@ class Juice < ::Test::Unit::TestCase
     assert_equal('null', json)
   end
   def test_json_object_compat
-    Oj.default_options = { :mode => :compat }
+    Oj.default_options = { :mode => :compat, :use_to_json => true }
     obj = Jeez.new(true, 58)
     json = Oj.dump(obj, :indent => 2)
-    assert(%{{"json_class":"Jeez","x":true,"y":58}} == json ||
-           %{{"json_class":"Jeez","y":58,"x":true}} == json)
+    assert(%{{"json_class":"Jeez","x":true,"y":58}
+} == json ||
+           %{{"json_class":"Jeez","y":58,"x":true}
+} == json)
     dump_and_load(obj, false)
+    Oj.default_options = { :mode => :compat, :use_to_json => false }
   end
   def test_json_object_create_id
     Oj.default_options = { :mode => :compat, :create_id => 'kson_class' }
@@ -560,12 +592,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Jeez",
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Jeez",
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -575,10 +609,11 @@ class Juice < ::Test::Unit::TestCase
     obj = Jazz.new(true, 58)
     begin
       Oj.dump(obj, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_to_hash_object_null
     obj = Jazz.new(true, 58)
@@ -598,12 +633,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Jazz",
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Jazz",
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -613,10 +650,11 @@ class Juice < ::Test::Unit::TestCase
     obj = Orange.new(true, 58)
     begin
       Oj.dump(obj, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
 
   def test_as_json_object_null
@@ -647,12 +685,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Orange",
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Orange",
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -662,10 +702,11 @@ class Juice < ::Test::Unit::TestCase
     obj = Jam.new(true, 58)
     begin
       Oj.dump(obj, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_object_null
     obj = Jam.new(true, 58)
@@ -678,11 +719,13 @@ class Juice < ::Test::Unit::TestCase
     assert(%{{
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
   end
   def test_object_object
     obj = Jam.new(true, 58)
@@ -691,12 +734,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Jam",
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Jam",
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -708,12 +753,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Jam",
   "x":true,
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Jam",
   "y":58,
   "x":true
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object, :class_cache => false)
     assert_equal(obj, obj2)
   end
@@ -723,7 +770,6 @@ class Juice < ::Test::Unit::TestCase
     err = nil
     begin
       raise StandardError.new('A Message')
-      assert(false)
     rescue Exception => e
       err = e
     end
@@ -747,10 +793,11 @@ class Juice < ::Test::Unit::TestCase
   def test_range_strict
     begin
       Oj.dump(1..7, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_range_null
     json = Oj.dump(1..7, :mode => :null)
@@ -836,15 +883,15 @@ class Juice < ::Test::Unit::TestCase
     assert_equal(Float, bg.class)
     assert_equal(orig.to_f, bg)
   end
-  def test_bigdecimal_compat_to_json
+  def test_bigdecimal_compat_as_json
     orig = BigDecimal.new('80.51')
-    BigDecimal.send(:define_method, :to_json) do
-      %{"this is big"}
+    BigDecimal.send(:define_method, :as_json) do
+      %{this is big}
     end
     json = Oj.dump(orig, :mode => :compat)
     bg = Oj.load(json, :mode => :compat)
     assert_equal("this is big", bg)
-    BigDecimal.send(:remove_method, :to_json) # cleanup
+    BigDecimal.send(:remove_method, :as_json) # cleanup
   end
   def test_bigdecimal_object
     mode = Oj.default_options[:mode]
@@ -862,10 +909,11 @@ class Juice < ::Test::Unit::TestCase
   def test_date_strict
     begin
       Oj.dump(Date.new(2012, 6, 19), :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_date_null
     json = Oj.dump(Date.new(2012, 6, 19), :mode => :null)
@@ -891,10 +939,11 @@ class Juice < ::Test::Unit::TestCase
   def test_datetime_strict
     begin
       Oj.dump(DateTime.new(2012, 6, 19, 20, 19, 27), :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_datetime_null
     json = Oj.dump(DateTime.new(2012, 6, 19, 20, 19, 27), :mode => :null)
@@ -934,13 +983,15 @@ class Juice < ::Test::Unit::TestCase
   "^i":1,
   "x":"^r1",
   "y":58
-}} == json ||
+}
+} == json ||
 %{{
   "^o":"Jam",
   "^i":1,
   "y":58,
   "x":"^r1"
-}} == json)
+}
+} == json)
     obj2 = Oj.load(json, :mode => :object, :circular => true)
     assert_equal(obj2.x.__id__, obj2.__id__)
   end
@@ -963,7 +1014,8 @@ class Juice < ::Test::Unit::TestCase
   "^i1",
   7,
   "^r1"
-]}, json)
+]
+}, json)
     a2 = Oj.load(json, :mode => :object, :circular => true)
     assert_equal(a2[1].__id__, a2.__id__)
   end
@@ -991,6 +1043,18 @@ class Juice < ::Test::Unit::TestCase
     end
   end
 
+  def test_deep_nest_dump
+    begin
+      a = []
+      10000.times { a << [a] }
+      Oj.dump(a)
+    rescue Exception => e
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
 # Stream IO
   def test_io_string
     src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
@@ -1062,6 +1126,21 @@ class Juice < ::Test::Unit::TestCase
     assert_equal({ 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}, obj)
   end
 
+  def test_nilnil_false
+    begin
+      Oj.load(nil)
+    rescue Exception => e
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
+  def test_nilnil_true
+    obj = Oj.load(nil, :nilnil => true)
+    assert_equal(nil, obj)
+  end
+
 # Rails re-call test. Active support recalls the json dumper when the to_json
 # method is called. This mimics that and verifies Oj detects it.
   def test_rails_like
